@@ -1,6 +1,5 @@
 const char* mysql_fix_privilege_tables={
-"-- Copyright (c) 2007, 2008 MySQL AB, 2009 Sun Microsystems, Inc.\n "
-"-- Use is subject to license terms.\n "
+"-- Copyright (C) 2007, 2008 MySQL AB, 2009 Sun Microsystems, Inc.\n "
 "-- \n "
 "-- This program is free software; you can redistribute it and/or modify\n "
 "-- it under the terms of the GNU General Public License as published by\n "
@@ -13,7 +12,7 @@ const char* mysql_fix_privilege_tables={
 "-- \n "
 "-- You should have received a copy of the GNU General Public License\n "
 "-- along with this program; if not, write to the Free Software\n "
-"-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA\n "
+"-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n "
 "\n "
 "--\n "
 "-- The system tables of MySQL Server\n "
@@ -24,23 +23,24 @@ const char* mysql_fix_privilege_tables={
 "\n "
 "CREATE TABLE IF NOT EXISTS db (   Host char(60) binary DEFAULT '' NOT NULL, Db char(64) binary DEFAULT '' NOT NULL, User char(16) binary DEFAULT '' NOT NULL, Select_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Insert_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Update_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Delete_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT"
 " NULL, Drop_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Grant_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, References_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Index_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT "
-"'N' NOT NULL, Create_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Show_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Execute_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, PRIMARY KEY Host (Host,Db,User), KEY User (User) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin comment='Dat"
-"abase privileges';\n "
+"'N' NOT NULL, Create_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Show_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Execute_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Event_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Trigger_priv enum('N','Y') COLLATE utf8_ge"
+"neral_ci DEFAULT 'N' NOT NULL, PRIMARY KEY Host (Host,Db,User), KEY User (User) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin comment='Database privileges';\n "
 "\n "
 "-- Remember for later if db table already existed\n "
 "set @had_db_table= @@warning_count != 0;\n "
 "\n "
 "CREATE TABLE IF NOT EXISTS host (  Host char(60) binary DEFAULT '' NOT NULL, Db char(64) binary DEFAULT '' NOT NULL, Select_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Insert_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Update_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Delete_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Drop_priv enum('N','Y') COLLATE ut"
 "f8_general_ci DEFAULT 'N' NOT NULL, Grant_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, References_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Index_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_view_priv enum('N','"
-"Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Show_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Execute_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, PRIMARY KEY Host (Host,Db) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin comment='Host privileges;  Merged with database privileges';\n "
+"Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Show_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Execute_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, PRIMARY KEY Host (Host,Db) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin co"
+"mment='Host privileges;  Merged with database privileges';\n "
 "\n "
 "\n "
 "CREATE TABLE IF NOT EXISTS user (   Host char(60) binary DEFAULT '' NOT NULL, User char(16) binary DEFAULT '' NOT NULL, Password char(41) character set latin1 collate latin1_bin DEFAULT '' NOT NULL, Select_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Insert_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Update_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Delete_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_priv enum('N','Y'"
 ") COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Drop_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Reload_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Shutdown_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Process_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, File_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Grant_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, References_priv enum('N','Y') COL"
 "LATE utf8_general_ci DEFAULT 'N' NOT NULL, Index_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Show_db_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Super_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Execute_priv enum('N','"
 "Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Repl_slave_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Repl_client_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Show_view_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Create_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Alter_routine_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NUL"
-"L, Create_user_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, ssl_type enum('','ANY','X509', 'SPECIFIED') COLLATE utf8_general_ci DEFAULT '' NOT NULL, ssl_cipher BLOB NOT NULL, x509_issuer BLOB NOT NULL, x509_subject BLOB NOT NULL, max_questions int(11) unsigned DEFAULT 0  NOT NULL, max_updates int(11) unsigned DEFAULT 0  NOT NULL, max_connections int(11) unsigned DEFAULT 0  NOT NULL, max_user_connections int(11) unsigned DEFAULT 0  NOT NULL, PRIMARY KEY Host (Host,User) ) engine=MyISAM C"
-"HARACTER SET utf8 COLLATE utf8_bin comment='Users and global privileges';\n "
+"L, Create_user_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Event_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL, ssl_type enum('','ANY','X509', 'SPECIFIED') COLLATE utf8_general_ci DEFAULT '' NOT NULL, ssl_cipher BLOB NOT NULL, x509_issuer BLOB NOT NULL, x509_subject BLOB NOT NULL, max_questions int(11) unsigned DEFAULT 0  NOT NULL, max_updates int(11) unsigned DEFAULT 0  NOT NULL, max_connections"
+" int(11) unsigned DEFAULT 0  NOT NULL, max_user_connections int(11) unsigned DEFAULT 0  NOT NULL, PRIMARY KEY Host (Host,User) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin comment='Users and global privileges';\n "
 "\n "
 "-- Remember for later if user table already existed\n "
 "set @had_user_table= @@warning_count != 0;\n "
@@ -49,8 +49,14 @@ const char* mysql_fix_privilege_tables={
 "CREATE TABLE IF NOT EXISTS func (  name char(64) binary DEFAULT '' NOT NULL, ret tinyint(1) DEFAULT '0' NOT NULL, dl char(128) DEFAULT '' NOT NULL, type enum ('function','aggregate') COLLATE utf8_general_ci NOT NULL, PRIMARY KEY (name) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin   comment='User defined functions';\n "
 "\n "
 "\n "
-"CREATE TABLE IF NOT EXISTS tables_priv ( Host char(60) binary DEFAULT '' NOT NULL, Db char(64) binary DEFAULT '' NOT NULL, User char(16) binary DEFAULT '' NOT NULL, Table_name char(64) binary DEFAULT '' NOT NULL, Grantor char(77) DEFAULT '' NOT NULL, Timestamp timestamp(14), Table_priv set('Select','Insert','Update','Delete','Create','Drop','Grant','References','Index','Alter','Create View','Show view') COLLATE utf8_general_ci DEFAULT '' NOT NULL, Column_priv set('Select','Insert','Update','References') CO"
-"LLATE utf8_general_ci DEFAULT '' NOT NULL, PRIMARY KEY (Host,Db,User,Table_name), KEY Grantor (Grantor) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin   comment='Table privileges';\n "
+"CREATE TABLE IF NOT EXISTS plugin ( name char(64) binary DEFAULT '' NOT NULL, dl char(128) DEFAULT '' NOT NULL, PRIMARY KEY (name) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin comment='MySQL plugins';\n "
+"\n "
+"\n "
+"CREATE TABLE IF NOT EXISTS servers ( Server_name char(64) NOT NULL DEFAULT '', Host char(64) NOT NULL DEFAULT '', Db char(64) NOT NULL DEFAULT '', Username char(64) NOT NULL DEFAULT '', Password char(64) NOT NULL DEFAULT '', Port INT(4) NOT NULL DEFAULT '0', Socket char(64) NOT NULL DEFAULT '', Wrapper char(64) NOT NULL DEFAULT '', Owner char(64) NOT NULL DEFAULT '', PRIMARY KEY (Server_name)) CHARACTER SET utf8 comment='MySQL Foreign Servers table';\n "
+"\n "
+"\n "
+"CREATE TABLE IF NOT EXISTS tables_priv ( Host char(60) binary DEFAULT '' NOT NULL, Db char(64) binary DEFAULT '' NOT NULL, User char(16) binary DEFAULT '' NOT NULL, Table_name char(64) binary DEFAULT '' NOT NULL, Grantor char(77) DEFAULT '' NOT NULL, Timestamp timestamp(14), Table_priv set('Select','Insert','Update','Delete','Create','Drop','Grant','References','Index','Alter','Create View','Show view','Trigger') COLLATE utf8_general_ci DEFAULT '' NOT NULL, Column_priv set('Select','Insert','Update','Refer"
+"ences') COLLATE utf8_general_ci DEFAULT '' NOT NULL, PRIMARY KEY (Host,Db,User,Table_name), KEY Grantor (Grantor) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin   comment='Table privileges';\n "
 "\n "
 "CREATE TABLE IF NOT EXISTS columns_priv ( Host char(60) binary DEFAULT '' NOT NULL, Db char(64) binary DEFAULT '' NOT NULL, User char(16) binary DEFAULT '' NOT NULL, Table_name char(64) binary DEFAULT '' NOT NULL, Column_name char(64) binary DEFAULT '' NOT NULL, Timestamp timestamp(14), Column_priv set('Select','Insert','Update','References') COLLATE utf8_general_ci DEFAULT '' NOT NULL, PRIMARY KEY (Host,Db,User,Table_name,Column_name) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin   comment='Column p"
 "rivileges';\n "
@@ -83,16 +89,39 @@ const char* mysql_fix_privilege_tables={
 "CREATE TABLE IF NOT EXISTS time_zone_leap_second (   Transition_time bigint signed NOT NULL, Correction int signed NOT NULL, PRIMARY KEY TranTime (Transition_time) ) engine=MyISAM CHARACTER SET utf8   comment='Leap seconds information for time zones';\n "
 "\n "
 "\n "
-"CREATE TABLE IF NOT EXISTS proc ( db char(64) collate utf8_bin DEFAULT '' NOT NULL, name  char(64) DEFAULT '' NOT NULL, type  enum('FUNCTION','PROCEDURE') NOT NULL, specific_name char(64) DEFAULT '' NOT NULL, language  enum('SQL') DEFAULT 'SQL' NOT NULL, sql_data_access enum('CONTAINS_SQL', 'NO_SQL', 'READS_SQL_DATA', 'MODIFIES_SQL_DATA' ) DEFAULT 'CONTAINS_SQL' NOT NULL, is_deterministic  enum('YES','NO') DEFAULT 'NO' NOT NULL, security_type enum('INVOKER','DEFINER') DEFAULT 'DEFINER' NOT NULL, param_list"
-"  blob NOT NULL, returns char(64) DEFAULT '' NOT NULL, body  longblob NOT NULL, definer char(77) collate utf8_bin DEFAULT '' NOT NULL, created timestamp, modified timestamp, sql_mode set( 'REAL_AS_FLOAT', 'PIPES_AS_CONCAT', 'ANSI_QUOTES', 'IGNORE_SPACE', 'NOT_USED', 'ONLY_FULL_GROUP_BY', 'NO_UNSIGNED_SUBTRACTION', 'NO_DIR_IN_CREATE', 'POSTGRESQL', 'ORACLE', 'MSSQL', 'DB2', 'MAXDB', 'NO_KEY_OPTIONS', 'NO_TABLE_OPTIONS', 'NO_FIELD_OPTIONS', 'MYSQL323', 'MYSQL40', 'ANSI', 'NO_AUTO_VALUE_ON_ZERO', 'NO_BACKSLAS"
-"H_ESCAPES', 'STRICT_TRANS_TABLES', 'STRICT_ALL_TABLES', 'NO_ZERO_IN_DATE', 'NO_ZERO_DATE', 'INVALID_DATES', 'ERROR_FOR_DIVISION_BY_ZERO', 'TRADITIONAL', 'NO_AUTO_CREATE_USER', 'HIGH_NOT_PRECEDENCE' ) DEFAULT '' NOT NULL, comment char(64) collate utf8_bin DEFAULT '' NOT NULL, PRIMARY KEY (db,name,type) ) engine=MyISAM character set utf8 comment='Stored Procedures';\n "
-"\n "
+"CREATE TABLE IF NOT EXISTS proc (db char(64) collate utf8_bin DEFAULT '' NOT NULL, name char(64) DEFAULT '' NOT NULL, type enum('FUNCTION','PROCEDURE') NOT NULL, specific_name char(64) DEFAULT '' NOT NULL, language enum('SQL') DEFAULT 'SQL' NOT NULL, sql_data_access enum( 'CONTAINS_SQL', 'NO_SQL', 'READS_SQL_DATA', 'MODIFIES_SQL_DATA') DEFAULT 'CONTAINS_SQL' NOT NULL, is_deterministic enum('YES','NO') DEFAULT 'NO' NOT NULL, security_type enum('INVOKER','DEFINER') DEFAULT 'DEFINER' NOT NULL, param_list blob"
+" NOT NULL, returns longblob DEFAULT '' NOT NULL, body longblob NOT NULL, definer char(77) collate utf8_bin DEFAULT '' NOT NULL, created timestamp, modified timestamp, sql_mode set( 'REAL_AS_FLOAT', 'PIPES_AS_CONCAT', 'ANSI_QUOTES', 'IGNORE_SPACE', 'NOT_USED', 'ONLY_FULL_GROUP_BY', 'NO_UNSIGNED_SUBTRACTION', 'NO_DIR_IN_CREATE', 'POSTGRESQL', 'ORACLE', 'MSSQL', 'DB2', 'MAXDB', 'NO_KEY_OPTIONS', 'NO_TABLE_OPTIONS', 'NO_FIELD_OPTIONS', 'MYSQL323', 'MYSQL40', 'ANSI', 'NO_AUTO_VALUE_ON_ZERO', 'NO_BACKSLASH_ESCAP"
+"ES', 'STRICT_TRANS_TABLES', 'STRICT_ALL_TABLES', 'NO_ZERO_IN_DATE', 'NO_ZERO_DATE', 'INVALID_DATES', 'ERROR_FOR_DIVISION_BY_ZERO', 'TRADITIONAL', 'NO_AUTO_CREATE_USER', 'HIGH_NOT_PRECEDENCE', 'NO_ENGINE_SUBSTITUTION', 'PAD_CHAR_TO_FULL_LENGTH') DEFAULT '' NOT NULL, comment char(64) collate utf8_bin DEFAULT '' NOT NULL, character_set_client char(32) collate utf8_bin, collation_connection char(32) collate utf8_bin, db_collation char(32) collate utf8_bin, body_utf8 longblob, PRIMARY KEY (db,name,type)) engine"
+"=MyISAM character set utf8 comment='Stored Procedures';\n "
 "\n "
 "CREATE TABLE IF NOT EXISTS procs_priv ( Host char(60) binary DEFAULT '' NOT NULL, Db char(64) binary DEFAULT '' NOT NULL, User char(16) binary DEFAULT '' NOT NULL, Routine_name char(64) COLLATE utf8_general_ci DEFAULT '' NOT NULL, Routine_type enum('FUNCTION','PROCEDURE') NOT NULL, Grantor char(77) DEFAULT '' NOT NULL, Proc_priv set('Execute','Alter Routine','Grant') COLLATE utf8_general_ci DEFAULT '' NOT NULL, Timestamp timestamp(14), PRIMARY KEY (Host,Db,User,Routine_name,Routine_type), KEY Grantor (Gran"
 "tor) ) engine=MyISAM CHARACTER SET utf8 COLLATE utf8_bin   comment='Procedure privileges';\n "
 "\n "
-"# Copyright (c) 2003, 2007 MySQL AB, 2009 Sun Microsystems, Inc.\n "
-"# Use is subject to license terms.\n "
+"-- Create general_log if CSV is enabled.\n "
+"\n "
+"SET @str = IF (@@have_csv = 'YES', 'CREATE TABLE IF NOT EXISTS general_log (event_time TIMESTAMP NOT NULL, user_host MEDIUMTEXT NOT NULL, thread_id INTEGER NOT NULL, server_id INTEGER UNSIGNED NOT NULL, command_type VARCHAR(64) NOT NULL, argument MEDIUMTEXT NOT NULL) engine=CSV CHARACTER SET utf8 comment=\"General log\"', 'SET @dummy = 0');\n "
+"\n "
+"PREPARE stmt FROM @str;\n "
+"EXECUTE stmt;\n "
+"DROP PREPARE stmt;\n "
+"\n "
+"-- Create slow_log if CSV is enabled.\n "
+"\n "
+"SET @str = IF (@@have_csv = 'YES', 'CREATE TABLE IF NOT EXISTS slow_log (start_time TIMESTAMP NOT NULL, user_host MEDIUMTEXT NOT NULL, query_time TIME NOT NULL, lock_time TIME NOT NULL, rows_sent INTEGER NOT NULL, rows_examined INTEGER NOT NULL, db VARCHAR(512) NOT NULL, last_insert_id INTEGER NOT NULL, insert_id INTEGER NOT NULL, server_id INTEGER UNSIGNED NOT NULL, sql_text MEDIUMTEXT NOT NULL) engine=CSV CHARACTER SET utf8 comment=\"Slow log\"', 'SET @dummy = 0');\n "
+"\n "
+"PREPARE stmt FROM @str;\n "
+"EXECUTE stmt;\n "
+"DROP PREPARE stmt;\n "
+"\n "
+"CREATE TABLE IF NOT EXISTS event ( db char(64) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL default '', name char(64) CHARACTER SET utf8 NOT NULL default '', body longblob NOT NULL, definer char(77) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL default '', execute_at DATETIME default NULL, interval_value int(11) default NULL, interval_field ENUM('YEAR','QUARTER','MONTH','DAY','HOUR','MINUTE','WEEK','SECOND','MICROSECOND','YEAR_MONTH','DAY_HOUR','DAY_MINUTE','DAY_SECOND','HOUR_MINUTE','HOUR_SECOND','MINUTE_S"
+"ECOND','DAY_MICROSECOND','HOUR_MICROSECOND','MINUTE_MICROSECOND','SECOND_MICROSECOND') default NULL, created TIMESTAMP NOT NULL, modified TIMESTAMP NOT NULL, last_executed DATETIME default NULL, starts DATETIME default NULL, ends DATETIME default NULL, status ENUM('ENABLED','DISABLED','SLAVESIDE_DISABLED') NOT NULL default 'ENABLED', on_completion ENUM('DROP','PRESERVE') NOT NULL default 'DROP', sql_mode  set('REAL_AS_FLOAT','PIPES_AS_CONCAT','ANSI_QUOTES','IGNORE_SPACE','NOT_USED','ONLY_FULL_GROUP_BY','NO"
+"_UNSIGNED_SUBTRACTION','NO_DIR_IN_CREATE','POSTGRESQL','ORACLE','MSSQL','DB2','MAXDB','NO_KEY_OPTIONS','NO_TABLE_OPTIONS','NO_FIELD_OPTIONS','MYSQL323','MYSQL40','ANSI','NO_AUTO_VALUE_ON_ZERO','NO_BACKSLASH_ESCAPES','STRICT_TRANS_TABLES','STRICT_ALL_TABLES','NO_ZERO_IN_DATE','NO_ZERO_DATE','INVALID_DATES','ERROR_FOR_DIVISION_BY_ZERO','TRADITIONAL','NO_AUTO_CREATE_USER','HIGH_NOT_PRECEDENCE','NO_ENGINE_SUBSTITUTION','PAD_CHAR_TO_FULL_LENGTH') DEFAULT '' NOT NULL, comment char(64) CHARACTER SET utf8 COLLATE "
+"utf8_bin NOT NULL default '', originator INTEGER UNSIGNED NOT NULL, time_zone char(64) CHARACTER SET latin1 NOT NULL DEFAULT 'SYSTEM', character_set_client char(32) collate utf8_bin, collation_connection char(32) collate utf8_bin, db_collation char(32) collate utf8_bin, body_utf8 longblob, PRIMARY KEY (db, name) ) ENGINE=MyISAM DEFAULT CHARSET=utf8 COMMENT 'Events';\n "
+"\n "
+"\n "
+"CREATE TABLE IF NOT EXISTS ndb_binlog_index (Position BIGINT UNSIGNED NOT NULL, File VARCHAR(255) NOT NULL, epoch BIGINT UNSIGNED NOT NULL, inserts BIGINT UNSIGNED NOT NULL, updates BIGINT UNSIGNED NOT NULL, deletes BIGINT UNSIGNED NOT NULL, schemaops BIGINT UNSIGNED NOT NULL, PRIMARY KEY(epoch)) ENGINE=MYISAM;\n "
+"\n "
+"# Copyright (C) 2003, 2007 MySQL AB, 2009 Sun Microsystems, Inc.\n "
 "# \n "
 "# This program is free software; you can redistribute it and/or modify\n "
 "# it under the terms of the GNU General Public License as published by\n "
@@ -105,7 +134,7 @@ const char* mysql_fix_privilege_tables={
 "# \n "
 "# You should have received a copy of the GNU General Public License\n "
 "# along with this program; if not, write to the Free Software\n "
-"# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA\n "
+"# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n "
 "\n "
 "# This part converts any old privilege tables to privilege tables suitable\n "
 "# for current version of MySQL\n "
@@ -150,7 +179,6 @@ const char* mysql_fix_privilege_tables={
 "#\n "
 "# tables_priv\n "
 "#\n "
-"\n "
 "ALTER TABLE tables_priv\n "
 "  ADD KEY Grantor (Grantor);\n "
 "\n "
@@ -168,15 +196,16 @@ const char* mysql_fix_privilege_tables={
 "    COLLATE utf8_general_ci DEFAULT '' NOT NULL,\n "
 "  MODIFY Table_priv set('Select','Insert','Update','Delete','Create',\n "
 "                        'Drop','Grant','References','Index','Alter',\n "
-"                        'Create View','Show view')\n "
+"                        'Create View','Show view','Trigger')\n "
 "    COLLATE utf8_general_ci DEFAULT '' NOT NULL,\n "
 "  COMMENT='Table privileges';\n "
 "\n "
 "#\n "
 "# columns_priv\n "
 "#\n "
-"\n "
+"#\n "
 "# Name change of Type -> Column_priv from MySQL 3.22.12\n "
+"#\n "
 "ALTER TABLE columns_priv\n "
 "  CHANGE Type Column_priv set('Select','Insert','Update','References')\n "
 "    COLLATE utf8_general_ci DEFAULT '' NOT NULL;\n "
@@ -240,8 +269,8 @@ const char* mysql_fix_privilege_tables={
 "ADD Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,\n "
 "ADD Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;\n "
 "ALTER TABLE host\n "
-"ADD Create_tmp_table_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL,\n "
-"ADD Lock_tables_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;\n "
+"ADD Create_tmp_table_priv enum('N','Y') DEFAULT 'N' NOT NULL,\n "
+"ADD Lock_tables_priv enum('N','Y') DEFAULT 'N' NOT NULL;\n "
 "\n "
 "alter table user change max_questions max_questions int(11) unsigned DEFAULT 0  NOT NULL;\n "
 "\n "
@@ -325,6 +354,37 @@ const char* mysql_fix_privilege_tables={
 "  MODIFY type enum ('function','aggregate') COLLATE utf8_general_ci NOT NULL;\n "
 "\n "
 "#\n "
+"# Modify log tables.\n "
+"#\n "
+"\n "
+"SET @old_log_state = @@global.general_log;\n "
+"SET GLOBAL general_log = 'OFF';\n "
+"ALTER TABLE general_log\n "
+"  MODIFY event_time TIMESTAMP NOT NULL,\n "
+"  MODIFY user_host MEDIUMTEXT NOT NULL,\n "
+"  MODIFY thread_id INTEGER NOT NULL,\n "
+"  MODIFY server_id INTEGER UNSIGNED NOT NULL,\n "
+"  MODIFY command_type VARCHAR(64) NOT NULL,\n "
+"  MODIFY argument MEDIUMTEXT NOT NULL;\n "
+"SET GLOBAL general_log = @old_log_state;\n "
+"\n "
+"SET @old_log_state = @@global.slow_query_log;\n "
+"SET GLOBAL slow_query_log = 'OFF';\n "
+"ALTER TABLE slow_log\n "
+"  MODIFY start_time TIMESTAMP NOT NULL,\n "
+"  MODIFY user_host MEDIUMTEXT NOT NULL,\n "
+"  MODIFY query_time TIME NOT NULL,\n "
+"  MODIFY lock_time TIME NOT NULL,\n "
+"  MODIFY rows_sent INTEGER NOT NULL,\n "
+"  MODIFY rows_examined INTEGER NOT NULL,\n "
+"  MODIFY db VARCHAR(512) NOT NULL,\n "
+"  MODIFY last_insert_id INTEGER NOT NULL,\n "
+"  MODIFY insert_id INTEGER NOT NULL,\n "
+"  MODIFY server_id INTEGER UNSIGNED NOT NULL,\n "
+"  MODIFY sql_text MEDIUMTEXT NOT NULL;\n "
+"SET GLOBAL slow_query_log = @old_log_state;\n "
+"\n "
+"#\n "
 "# Detect whether we had Create_view_priv\n "
 "#\n "
 "SET @hadCreateViewPriv:=0;\n "
@@ -456,7 +516,8 @@ const char* mysql_fix_privilege_tables={
 "                             'READS_SQL_DATA',\n "
 "                             'MODIFIES_SQL_DATA'\n "
 "                            ) DEFAULT 'CONTAINS_SQL' NOT NULL,\n "
-"                 MODIFY body longblob DEFAULT '' NOT NULL,\n "
+"                 MODIFY body longblob NOT NULL,\n "
+"                 MODIFY returns longblob NOT NULL,\n "
 "                 MODIFY sql_mode\n "
 "                        set('REAL_AS_FLOAT',\n "
 "                            'PIPES_AS_CONCAT',\n "
@@ -487,7 +548,9 @@ const char* mysql_fix_privilege_tables={
 "                            'ERROR_FOR_DIVISION_BY_ZERO',\n "
 "                            'TRADITIONAL',\n "
 "                            'NO_AUTO_CREATE_USER',\n "
-"                            'HIGH_NOT_PRECEDENCE'\n "
+"                            'HIGH_NOT_PRECEDENCE',\n "
+"                            'NO_ENGINE_SUBSTITUTION',\n "
+"                            'PAD_CHAR_TO_FULL_LENGTH'\n "
 "                            ) DEFAULT '' NOT NULL,\n "
 "                 DEFAULT CHARACTER SET utf8;\n "
 "\n "
@@ -501,6 +564,166 @@ const char* mysql_fix_privilege_tables={
 "                  MODIFY comment\n "
 "                         char(64) collate utf8_bin DEFAULT '' NOT NULL;\n "
 "\n "
+"ALTER TABLE proc ADD character_set_client\n "
+"                     char(32) collate utf8_bin DEFAULT NULL\n "
+"                     AFTER comment;\n "
+"ALTER TABLE proc MODIFY character_set_client\n "
+"                        char(32) collate utf8_bin DEFAULT NULL;\n "
+"\n "
+"SELECT CASE WHEN COUNT(*) > 0 THEN \n "
+"CONCAT (\"WARNING: NULL values of the 'character_set_client' column ('mysql.proc' table) have been updated with a default value (\", @@character_set_client, \"). Please verify if necessary.\")\n "
+"ELSE NULL \n "
+"END \n "
+"AS value FROM proc WHERE character_set_client IS NULL;\n "
+"\n "
+"UPDATE proc SET character_set_client = @@character_set_client \n "
+"                     WHERE character_set_client IS NULL;\n "
+"\n "
+"ALTER TABLE proc ADD collation_connection\n "
+"                     char(32) collate utf8_bin DEFAULT NULL\n "
+"                     AFTER character_set_client;\n "
+"ALTER TABLE proc MODIFY collation_connection\n "
+"                        char(32) collate utf8_bin DEFAULT NULL;\n "
+"\n "
+"SELECT CASE WHEN COUNT(*) > 0 THEN \n "
+"CONCAT (\"WARNING: NULL values of the 'collation_connection' column ('mysql.proc' table) have been updated with a default value (\", @@collation_connection, \"). Please verify if necessary.\")\n "
+"ELSE NULL \n "
+"END \n "
+"AS value FROM proc WHERE collation_connection IS NULL;\n "
+"\n "
+"UPDATE proc SET collation_connection = @@collation_connection\n "
+"                     WHERE collation_connection IS NULL;\n "
+"\n "
+"ALTER TABLE proc ADD db_collation\n "
+"                     char(32) collate utf8_bin DEFAULT NULL\n "
+"                     AFTER collation_connection;\n "
+"ALTER TABLE proc MODIFY db_collation\n "
+"                        char(32) collate utf8_bin DEFAULT NULL;\n "
+"\n "
+"SELECT CASE WHEN COUNT(*) > 0 THEN \n "
+"CONCAT (\"WARNING: NULL values of the 'db_collation' column ('mysql.proc' table) have been updated with default values. Please verify if necessary.\")\n "
+"ELSE NULL\n "
+"END\n "
+"AS value FROM proc WHERE db_collation IS NULL;\n "
+"\n "
+"UPDATE proc AS p SET db_collation  = \n "
+"                     ( SELECT DEFAULT_COLLATION_NAME \n "
+"                       FROM INFORMATION_SCHEMA.SCHEMATA \n "
+"                       WHERE SCHEMA_NAME = p.db)\n "
+"                     WHERE db_collation IS NULL;\n "
+"\n "
+"ALTER TABLE proc ADD body_utf8 longblob DEFAULT NULL\n "
+"                     AFTER db_collation;\n "
+"ALTER TABLE proc MODIFY body_utf8 longblob DEFAULT NULL;\n "
+"\n "
+"\n "
+"#\n "
+"# EVENT privilege\n "
+"#\n "
+"SET @hadEventPriv := 0;\n "
+"SELECT @hadEventPriv :=1 FROM user WHERE Event_priv LIKE '%';\n "
+"\n "
+"ALTER TABLE user add Event_priv enum('N','Y') character set utf8 DEFAULT 'N' NOT NULL AFTER Create_user_priv;\n "
+"ALTER TABLE user MODIFY Event_priv enum('N','Y') character set utf8 DEFAULT 'N' NOT NULL AFTER Create_user_priv;\n "
+"\n "
+"UPDATE user SET Event_priv=Super_priv WHERE @hadEventPriv = 0;\n "
+"\n "
+"ALTER TABLE db add Event_priv enum('N','Y') character set utf8 DEFAULT 'N' NOT NULL;\n "
+"ALTER TABLE db MODIFY Event_priv enum('N','Y') character set utf8 DEFAULT 'N' NOT NULL;\n "
+"\n "
+"#\n "
+"# EVENT table\n "
+"#\n "
+"ALTER TABLE event DROP PRIMARY KEY;\n "
+"ALTER TABLE event ADD PRIMARY KEY(db, name);\n "
+"# Add sql_mode column just in case.\n "
+"ALTER TABLE event ADD sql_mode set ('NOT_USED') AFTER on_completion;\n "
+"# Update list of sql_mode values.\n "
+"ALTER TABLE event MODIFY sql_mode\n "
+"                        set('REAL_AS_FLOAT',\n "
+"                            'PIPES_AS_CONCAT',\n "
+"                            'ANSI_QUOTES',\n "
+"                            'IGNORE_SPACE',\n "
+"                            'NOT_USED',\n "
+"                            'ONLY_FULL_GROUP_BY',\n "
+"                            'NO_UNSIGNED_SUBTRACTION',\n "
+"                            'NO_DIR_IN_CREATE',\n "
+"                            'POSTGRESQL',\n "
+"                            'ORACLE',\n "
+"                            'MSSQL',\n "
+"                            'DB2',\n "
+"                            'MAXDB',\n "
+"                            'NO_KEY_OPTIONS',\n "
+"                            'NO_TABLE_OPTIONS',\n "
+"                            'NO_FIELD_OPTIONS',\n "
+"                            'MYSQL323',\n "
+"                            'MYSQL40',\n "
+"                            'ANSI',\n "
+"                            'NO_AUTO_VALUE_ON_ZERO',\n "
+"                            'NO_BACKSLASH_ESCAPES',\n "
+"                            'STRICT_TRANS_TABLES',\n "
+"                            'STRICT_ALL_TABLES',\n "
+"                            'NO_ZERO_IN_DATE',\n "
+"                            'NO_ZERO_DATE',\n "
+"                            'INVALID_DATES',\n "
+"                            'ERROR_FOR_DIVISION_BY_ZERO',\n "
+"                            'TRADITIONAL',\n "
+"                            'NO_AUTO_CREATE_USER',\n "
+"                            'HIGH_NOT_PRECEDENCE',\n "
+"                            'NO_ENGINE_SUBSTITUTION',\n "
+"                            'PAD_CHAR_TO_FULL_LENGTH'\n "
+"                            ) DEFAULT '' NOT NULL AFTER on_completion;\n "
+"ALTER TABLE event MODIFY name char(64) CHARACTER SET utf8 NOT NULL default '';\n "
+"\n "
+"ALTER TABLE event MODIFY COLUMN originator INT UNSIGNED NOT NULL;\n "
+"ALTER TABLE event ADD COLUMN originator INT UNSIGNED NOT NULL AFTER comment;\n "
+"\n "
+"ALTER TABLE event MODIFY COLUMN status ENUM('ENABLED','DISABLED','SLAVESIDE_DISABLED') NOT NULL default 'ENABLED';\n "
+"\n "
+"ALTER TABLE event ADD COLUMN time_zone char(64) CHARACTER SET latin1\n "
+"        NOT NULL DEFAULT 'SYSTEM' AFTER originator;\n "
+"\n "
+"ALTER TABLE event ADD character_set_client\n "
+"                      char(32) collate utf8_bin DEFAULT NULL\n "
+"                      AFTER time_zone;\n "
+"ALTER TABLE event MODIFY character_set_client\n "
+"                         char(32) collate utf8_bin DEFAULT NULL;\n "
+"\n "
+"ALTER TABLE event ADD collation_connection\n "
+"                      char(32) collate utf8_bin DEFAULT NULL\n "
+"                      AFTER character_set_client;\n "
+"ALTER TABLE event MODIFY collation_connection\n "
+"                         char(32) collate utf8_bin DEFAULT NULL;\n "
+"\n "
+"ALTER TABLE event ADD db_collation\n "
+"                      char(32) collate utf8_bin DEFAULT NULL\n "
+"                      AFTER collation_connection;\n "
+"ALTER TABLE event MODIFY db_collation\n "
+"                         char(32) collate utf8_bin DEFAULT NULL;\n "
+"\n "
+"ALTER TABLE event ADD body_utf8 longblob DEFAULT NULL\n "
+"                      AFTER db_collation;\n "
+"ALTER TABLE event MODIFY body_utf8 longblob DEFAULT NULL;\n "
+"\n "
+"\n "
+"#\n "
+"# TRIGGER privilege\n "
+"#\n "
+"\n "
+"SET @hadTriggerPriv := 0;\n "
+"SELECT @hadTriggerPriv :=1 FROM user WHERE Trigger_priv LIKE '%';\n "
+"\n "
+"ALTER TABLE user ADD Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL AFTER Event_priv;\n "
+"ALTER TABLE user MODIFY Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL AFTER Event_priv;\n "
+"\n "
+"ALTER TABLE host ADD Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;\n "
+"ALTER TABLE host MODIFY Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;\n "
+"\n "
+"ALTER TABLE db ADD Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;\n "
+"ALTER TABLE db MODIFY Trigger_priv enum('N','Y') COLLATE utf8_general_ci DEFAULT 'N' NOT NULL;\n "
+"\n "
+"UPDATE user SET Trigger_priv=Super_priv WHERE @hadTriggerPriv = 0;\n "
+"\n "
 "# Activate the new, possible modified privilege tables\n "
 "# This should not be needed, but gives us some extra testing that the above\n "
 "# changes was correct\n "
