@@ -1,4 +1,4 @@
-/* Copyright (c) 2004, 2011, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2011, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
+   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
 #line 18 "decimal.c"
 
@@ -971,7 +971,7 @@ int decimal2double(decimal_t *from, double *to)
 
   *to= from->sign ? -result : result;
 
-  DBUG_PRINT("info", ("result: %f (%lx)", *to, *(ulong *)to));
+  DBUG_PRINT("info", ("result: %f", *to));
 
   return E_DEC_OK;
 }
@@ -991,12 +991,18 @@ int decimal2double(decimal_t *from, double *to)
 int double2decimal(double from, decimal_t *to)
 {
   /* TODO: fix it, when we'll have dtoa */
-  char s[400], *end;
-  sprintf(s, "%.16G", from);
-  end= strend(s);
-  return string2decimal(s, to, &end);
+  char buff[400], *end;
+  int length, res;
+  DBUG_ENTER("double2decimal");
+  length= sprintf(buff, "%.16G", from);
+  DBUG_PRINT("info",("from: %g  from_as_str: %s", from, buff));
+  end= buff+length;
+  res= string2decimal(buff, to, &end);
+  DBUG_PRINT("exit", ("res: %d", res));
+  DBUG_RETURN(res);
 }
 
+
 static int ull2dec(ulonglong from, decimal_t *to)
 {
   int intg1, error=E_DEC_OK;
@@ -1184,7 +1190,7 @@ int decimal2longlong(decimal_t *from, longlong *to)
 
                 7E F2 04 37 2D FB 2D
 */
-int decimal2bin(decimal_t *from, char *to, int precision, int frac)
+int decimal2bin(decimal_t *from, uchar *to, int precision, int frac)
 {
   dec1 mask=from->sign ? -1 : 0, *buf1=from->buf, *stop1;
   int error=E_DEC_OK, intg=precision-frac,
@@ -1200,7 +1206,7 @@ int decimal2bin(decimal_t *from, char *to, int precision, int frac)
       fsize1=frac1*sizeof(dec1)+dig2bytes[frac1x];
   const int orig_isize0= isize0;
   const int orig_fsize0= fsize0;
-  char *orig_to= to;
+  uchar *orig_to= to;
 
   buf1= remove_leading_zeroes(from, &from_intg);
 
@@ -1290,10 +1296,10 @@ int decimal2bin(decimal_t *from, char *to, int precision, int frac)
   }
   if (fsize0 > fsize1)
   {
-    char *to_end= orig_to + orig_fsize0 + orig_isize0;
+    uchar *to_end= orig_to + orig_fsize0 + orig_isize0;
 
     while (fsize0-- > fsize1 && to < to_end)
-      *to++=(uchar)mask;
+      *to++= (uchar)mask;
   }
   orig_to[0]^= 0x80;
 
@@ -1319,19 +1325,19 @@ int decimal2bin(decimal_t *from, char *to, int precision, int frac)
     E_DEC_OK/E_DEC_TRUNCATED/E_DEC_OVERFLOW
 */
 
-int bin2decimal(char *from, decimal_t *to, int precision, int scale)
+int bin2decimal(const uchar *from, decimal_t *to, int precision, int scale)
 {
   int error=E_DEC_OK, intg=precision-scale,
       intg0=intg/DIG_PER_DEC1, frac0=scale/DIG_PER_DEC1,
       intg0x=intg-intg0*DIG_PER_DEC1, frac0x=scale-frac0*DIG_PER_DEC1,
       intg1=intg0+(intg0x>0), frac1=frac0+(frac0x>0);
   dec1 *buf=to->buf, mask=(*from & 0x80) ? 0 : -1;
-  char *stop;
-  char *d_copy;
+  const uchar *stop;
+  uchar *d_copy;
   int bin_size= decimal_bin_size(precision, scale);
 
   sanity(to);
-  d_copy= (char *)my_alloca(bin_size);
+  d_copy= (uchar*) my_alloca(bin_size);
   memcpy(d_copy, from, bin_size);
   d_copy[0]^= 0x80;
   from= d_copy;
@@ -1479,8 +1485,9 @@ decimal_round(decimal_t *from, decimal_t *to, int scale,
 {
   int frac0=scale>0 ? ROUND_UP(scale) : scale/DIG_PER_DEC1,
     frac1=ROUND_UP(from->frac), UNINIT_VAR(round_digit),
-    intg0=ROUND_UP(from->intg), error=E_DEC_OK, len=to->len;
-
+      intg0=ROUND_UP(from->intg), error=E_DEC_OK, len=to->len,
+      intg1=ROUND_UP(from->intg +
+                     (((intg0 + frac0)>0) && (from->buf[0] == DIG_MAX)));
   dec1 *buf0=from->buf, *buf1=to->buf, x, y, carry=0;
   int first_dig;
 
@@ -1495,12 +1502,6 @@ decimal_round(decimal_t *from, decimal_t *to, int scale,
   default: DBUG_ASSERT(0);
   }
 
-  /*
-    For my_decimal we always use len == DECIMAL_BUFF_LENGTH == 9
-    For internal testing here (ifdef MAIN) we always use len == 100/4
-   */
-  DBUG_ASSERT(from->len == to->len);
-
   if (unlikely(frac0+intg0 > len))
   {
     frac0=len-intg0;
@@ -1514,17 +1515,17 @@ decimal_round(decimal_t *from, decimal_t *to, int scale,
     return E_DEC_OK;
   }
 
-  if (to != from)
+  if (to != from || intg1>intg0)
   {
     dec1 *p0= buf0+intg0+max(frac1, frac0);
-    dec1 *p1= buf1+intg0+max(frac1, frac0);
-
-    DBUG_ASSERT(p0 - buf0 <= len);
-    DBUG_ASSERT(p1 - buf1 <= len);
+    dec1 *p1= buf1+intg1+max(frac1, frac0);
 
     while (buf0 < p0)
       *(--p1) = *(--p0);
+    if (unlikely(intg1 > intg0))
+      to->buf[0]= 0;
 
+    intg0= intg1;
     buf0=to->buf;
     buf1=to->buf;
     to->sign=from->sign;
@@ -1933,8 +1934,7 @@ static int do_sub(decimal_t *from1, decimal_t *from2, decimal_t *to)
 int decimal_intg(decimal_t *from)
 {
   int res;
-  dec1 *tmp_res;
-  tmp_res= remove_leading_zeroes(from, &res);
+  remove_leading_zeroes(from, &res);
   return res;
 }
 
